#include <stdio.h>
#include <stdarg.h>
#include <assert.h>
#include "dptm2.h"

extern int ary_flags[];

/* display message on fp */
/* select showing or not by channels and flags */
int chprintf( int ch, FILE* fp, const char* str, ... )
{
	va_list args;
	int ierr = 0;

	assert( fp != NULL );

	switch( ch )
	{
	case 'D':
		//if( !g_flag_debug ) goto nomsg;
#ifdef NDEBUG
		goto nomsg;
#endif
		break;
	case 'F':   /* message: filenames */
		if( !ary_flags['L'] ) goto nomsg;
		break;
	case 'M':   /* message: memory allocation */
#ifdef NDEBUG
		goto nomsg;
#endif
		break;
	case 'N':   /* message: # of nodes */
		if( !ary_flags['L'] ) goto nomsg;
		break;
	case 'P':   /* message: page count */
		if( !ary_flags['L'] ) goto nomsg;
		break;
	default:
		fprintf( stderr, "Coding error: Next message has an undefined channel 0x%x\n", ch );
		ierr = 1;
		break;
	}

	va_start( args, str );
	vfprintf( fp, str, args );
	va_end( args );
nomsg:;

	assert( ierr == 0 );   /* stop running if wrong channel */
	return ierr;
}



/* display help */
void display_help( void )
{
	fputs( "Usage: dptm2 [options & files]\n", stderr );
	fputs( "        (or you can drag & drop PS files on this EXE file.)\n", stderr );
	fputs( "\n", stderr );
	fputs( "Options: usually, capital = ON / small = OFF\n"               , stderr );
	fputs( "    -T\n", stderr );
	fputs( "         Merge tones or not. (default -T)\n"                  , stderr );
	fputs( "    -B\n", stderr );
	fputs( "         Remove background or not. (default -b)\n"            , stderr );
	fputs( "    -W[p[/q]]\n", stderr );
	fputs( "         Change line width or not. (default -w)\n"            , stderr );
	fputs( "         You can specify convert parameters.\n"               , stderr );
	fputs( "             -W only -> index_new = 2.0 * index + 2.0\n"      , stderr );
	fputs( "             -Wp     -> index_new =  p  * index\n"            , stderr );
	fputs( "             -Wp/q   -> index_new =  p  * index +  q\n"       , stderr );
	fputs( "             where p and q are integers or floats.\n"         , stderr );
	fputs( "    -L[filename]\n", stderr );
	fputs( "         Display progress on STDERR. (default -l)\n"          , stderr );
	fputs( "         You can specify log file instead of STDERR.\n"       , stderr );
	fputs( "    -H/-h\n", stderr );
	fputs( "         Display help (this text).\n"                         , stderr );
	fputs( "    -V/-v\n", stderr );
	fputs( "         Display the version of this program.\n"              , stderr );
	fputs( "\n", stderr );
	fputs( "Files: Specify filename(s) you want to convert.\n"                 , stderr );
	fputs( "       Output name will be determined automatically\n"             , stderr );
	fputs( "          not to overwrite existing files.\n"                      , stderr );
	fputs( "       [example] input : hoge.ps\n"                                , stderr );
	fputs( "                 output: hoge_1.ps\n"                              , stderr );
	fputs( "       If there are no filenames, STDIN and STDOUT will be used.\n", stderr );
	fputs( "       This mode will catenate all the input files.\n"             , stderr );
	fputs( "\n", stderr );
}

/* display version */
void display_version( void )
{
	fputs( "Version: 1.00a  2013/05/16\n", stderr );
	fputs( "\n", stderr );
}
