#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "dptm2.h"

extern int ary_flags[];
extern FILE* fplog;
char fmt_out[16];
char fmt_sfx[16] = "_%d";

int sfxlen( void )
{
	int length;
	char buf[MAX_PATH];

	/* check for fmt_out overflow */
	assert( strnlen( fmt_sfx, numof( fmt_out ) ) <= numof( fmt_out ) - 1 - 4 );   /* '\0' & "%s"*2 */

	sprintf( buf, fmt_sfx, MAXSFXNUM );
	length = strnlen( buf, numof( buf ) );
	assert( length < numof( buf ) );

	return length;
}

int fopen_log( char* fname_log, BOOL append )
{
	assert( fplog == NULL );

	/* set fplog even if flag == FALSE */
	if( ary_flags['L'] && fname_log[0] != '\0' )
	{
		fplog = fopen( fname_log, ( append ? "a" : "w" ) );
		if( fplog == NULL )
		{
			fprintf( stderr, "Error: Failed to open log file %s\n", fname_log );
			return 1;
		}
	}
	else
	{
		fplog = stderr;
	}

	return 0;
}

void fclose_log( void )
{
	if( fplog != NULL )
	{
		if( fplog != stderr ) fclose( fplog );
		fplog = NULL;
	}

	return;
}

/* if you want to add to previous file, strlen(fname_out)>0 . */
int fopen_ps( const char fname_in[], char fname_out[], FILE** fppsin, FILE** fppsout )
{
	int ierr;
	int i, j;
	int len_fname_in;
	char ext[8];   /* long(>=7) extension is not allowed */
	char fname_base[MAX_PATH];
	BOOL append;


	assert( fplog != NULL );
	assert( *fppsin  == NULL );
	assert( *fppsout == NULL );

	ierr = 0;

	/* check PATH length */
	len_fname_in = strnlen( fname_in, MAX_PATH );
	if( len_fname_in > MAX_PATH - 1 - sfxlen() )   /* space for '\0' & suffix */
	{
		fprintf( stderr, "Error: fname_in is too long!\n" );
		return 1;
	}

	/* get basename and extension */
	strncpy2( fname_base, fname_in, numof( fname_base ) );
	ext[0] = '\0';
	for( i = strnlen( fname_in, numof( fname_base ) ), j = 0;
		 ( ( i >= 0 ) && ( j < numof( ext ) - 1 ) ); i--, j++ )
	{
		if( fname_in[i] == '.' )
		{
			fname_base[i] = '\0';
			strncpy2( ext, &fname_in[i], numof( ext ) );
			chprintf( 'D', stderr, "Debug: base = %s, ext = %s\n", fname_base, ext );
			break;
		}
	}

	/* make new filename */
	append = ( fname_out != NULL && fname_out[0] != '\0' );
	if( !append )
	{
		sprintf( fmt_out, "%%s%s%%s", fmt_sfx );   // (ex) fmt_out = "%s_%d%s"
		for( i = 1; i <= MAXSFXNUM; i++ )
		{
			sprintf( fname_out, fmt_out, fname_base, i, ext );
			if( !exist_file( fname_out ) ) break;
		}
		if( i > MAXSFXNUM )
		{
			fprintf( fplog, "Error: Failed to make new PS filename (last=%s)\n", fname_out );
			ierr = 1;   goto errhdl;
		}
	}

	/* open files */
	*fppsin = fopen( fname_in, "r" );
	if( *fppsin == NULL )
	{
		fprintf( fplog, "Error: Failed to open PSin  %s\n", fname_in );
		ierr = 1;   goto errhdl;
	}
	*fppsout = fopen( fname_out, ( append ? "a" : "w" ) );
	if( *fppsout == NULL )
	{
		fprintf( fplog, "Error: Failed to open PSout %s\n", fname_out );
		ierr = 1;   goto errhdl;
	}

	chprintf( 'F', fplog, "Input : %s\n", fname_in  );
	chprintf( 'F', fplog, "Output: %s\n", fname_out );

	return 0;

errhdl:
	fclose_ps( fppsin, fppsout );
	return ierr;
}

void fclose_ps( FILE** fppsin, FILE** fppsout )
{
	if( *fppsin  != NULL )
	{
		fclose( *fppsin  );
		*fppsin  = NULL;
	}
	if( *fppsout != NULL )
	{
		fclose( *fppsout );
		*fppsout = NULL;
	}

	return;
}
