#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "dptm2.h"

#define SZSTR_ALL "LMXTNA"

int ary_flags[96] = { 0 };   // include [A-Z]

int num_fname_in;
char** ary_fname_in;
char fname_out[MAX_PATH];   // rewritable
char* fname_log;       // reference
FILE* fplog;           // open/close in files.c

float width_coef[10][2];   // idx2=idx1*w1+w0
float width_psin;

int set_wc( char ch, float w1, float w0 )
{
	int wctype;
	char ch_upper;
	BOOL ch_flag;

	ch_upper = toupper( ch );
	ch_flag  = isupper( ch );

	switch( ch_upper )
	{
	case 'L':   /* polyline */
		wctype = 1;   break;
	case 'M':   /* polymarker */
		wctype = 2;   break;
	case 'X':   /* text */
		wctype = 3;   break;
	case 'T':   /* tone (soft-filled) */
		wctype = 4;   break;
	case 'N':   /* line */
		wctype = 5;   break;
	case 'A':   /* arrow */
		wctype = 6;   break;
	default:   /* error */
		fprintf( stderr, "Error: Invalid primitive [%c]!\n", ch );
		break;
	}

	/* channel OFF (v1.0) */
	wctype = 0;

	if( ary_flags['W'] && ch_flag )
	{
		width_coef[wctype][0] = w0;
		width_coef[wctype][1] = w1;
	}
	else
	{
		width_coef[wctype][0] = 0.0f;
		width_coef[wctype][1] = 1.0f;
	}

	return wctype;
}

int wc_debug( void )
{
	int i;

#ifndef NDEBUG
	chprintf( 'D', stderr, "Debug: width_coef list for ANY\n" );   /* ANY, PL, PM, TN, TX, LN, AL */
	chprintf( 'D', stderr, "      " );
	for( i = 0; i < 1; i++ )   /* numof( width_coef ) */
	{
		chprintf( 'D', stderr, " (%3.1f,%3.1f)", width_coef[i][1], width_coef[i][0] );
	}
	chprintf( 'D', stderr, "\n" );
#endif

	return 0;
}

int pickup_param1( char* buf )
{
	float param;
	char oprt[4];

	if( sscanf( buf, "%f %3s%*s", &param, &oprt[0] ) != 2 )
	{
		return 1;
	}

	/* save parameter (and overwrite buffer) */
	if( oprt[0] == 'W' )
	{
		/* setlinewidth */
		width_psin = param;
		sprintf( buf, "%4.1f W\n", param * width_coef[0][1] + width_coef[0][0] * 2.0f );
	}

	return 0;
}

int flags_readcmd( int argc, char* argv[] )
{
	int i, j;
	int ierr;

	/* initialize */
	ierr = 0;
	num_fname_in = 0;
	ary_fname_in = (char**)calloc( argc, sizeof(char*) );
	if( ary_fname_in == NULL )
	{
		fputs( "Error: Failed to allocate ary_fname_in.\n", stderr );
		return 1;
	}
	fname_out[0] = '\0';
	fname_log = "";
	for( i = 0; i < sizeof( width_coef ) / sizeof( width_coef[0] ); i++ )
	{
		width_coef[i][0] = 0.0f;
		width_coef[i][1] = 1.0f;
	}

	/* default flags */
	ary_flags['T'] = TRUE;
	fname_log = "";

	for( i = 1; i < argc; i++ )
	{
		if( argv[i][0] == '-' )
		{
			/* options */
			BOOL flag_upper;
			char flagkind;

			flag_upper = isupper( argv[i][1] );
			flagkind   = toupper( argv[i][1] );
			if( flagkind < 'A' || 'Z' < flagkind )
			{
				fprintf( stderr, "Error: Bad character option! : %s\n", argv[i] );
				ierr = 1;
				continue;
			}

			/* registrate */
			ary_flags[flagkind] = flag_upper;
			chprintf( 'D', stderr, "Option %c: %s\n", flagkind, L2STR( flag_upper ) );

			/* additional operations */
			switch( flagkind )
			{
			case 'H':   /* show help */
				ary_flags[flagkind] = TRUE;
				break;
			case 'L':   /* output log */
				fname_log = &argv[i][2];
				chprintf( 'D', stderr, "          log file = %s\n", ( fname_log[0] != '\0' ? fname_log : "(STDERR)" ) );
				break;
			case 'V':   /* show version */
				ary_flags[flagkind] = TRUE;
				break;
			case 'W':   /* change line width */
				{
					char str[16];   /* -W[LlMmXxTtNnAa] */
					int k;
					float a, b;
					float w1, w0;

					switch( sscanf( argv[i], " %15[^0-9]%f/%f", str, &a, &b ) )
					{
					case 3:
						w1 = a;      w0 = b;      break;
					case 2:
						w1 = a;      w0 = 0.0f;   break;
					default:
						w1 = 2.0f;   w0 = 2.0f;   break;
					}
					chprintf( 'D', stderr, "Debug: %s%f/%f\n", str, w1, w0 );

					if( str[2] == '\0' )
					{
						/* set same parameters for all sz-type */
						strncpy2( &str[2], SZSTR_ALL, numof( str ) - 2 );
						chprintf( 'D', stderr, "Debug: Update str to %s\n", str );
					}

					/* set parameters for specified sz-type */
					for( k = 2; ( k < numof( str ) && str[k] != '\0' ); k++ )
					{
						set_wc( str[k], w1, w0 );
					}

					wc_debug();
				}
				break;
			default:
				break;
			}
		}
		else
		{
			/* filenames */
			ary_fname_in[num_fname_in] = argv[i];
			num_fname_in++;
		}
	}

	/* help & version */
	if( ary_flags['H'] || ary_flags['V'] )
	{
		if( ary_flags['H'] ) display_help();
		if( ary_flags['V'] ) display_version();

		free( ary_fname_in );
		exit( EXIT_SUCCESS );
	}

	if( num_fname_in > 0 )
	{
		/* filename-mode */
		if( ary_flags['A'] )   /* Caution: this mode is not supported in v1.0 */
		{
			fprintf( stderr, "Error: Append mode (-A) cannot be used with filename specification.\n" );
			fprintf( stderr, "       Use redirection.\n" );
			ierr = 1;
		}
	}
	else
	{
		/* pipeline-mode */
		ary_flags['C'] = TRUE;
		chprintf( 'D', stderr, "Debug: No filenames.  Cat mode ON.\n" );
	}

	/* error handle */
	if( ierr != 0 )
	{
		flags_free();
	}

	return ierr;
}

int flags_waitclose( void )
{
#ifdef WAITCLOSE
	if( ary_flags['L'] && fname_log[0] == '\0' )
	{
		fprintf( stderr, "Finish program.  Press ENTER.\n" );
		if( fscanf( stdin, "%*[^\n]" ) == EOF ) return EOF;   /* any strings */
		if( fscanf( stdin, "%*c"     ) == EOF ) return EOF;   /* '\n' */
	}
#endif

	return 0;
}

void flags_free( void )
{
	num_fname_in = 0;
	if( ary_fname_in != NULL )
	{
		free( ary_fname_in );
		ary_fname_in = NULL;
	}

	return;
}

#undef SZSTR_ALL
