#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "dptm2.h"

extern FILE* fplog;
extern s_Node** ary2;   // for assert only
extern int ary_flags[];

static int count_page;
static char psheader[PSHEADMAX];
int block_sztn( FILE* fpin, FILE* fpout, char* buf_begin );
int block_else( FILE* fpin, FILE* fpout, char* buf_begin, char* str_end, BOOL skip );

BOOL strheadequal( const char* str1, const char* str2 )
{
	/* str2 should be literal */
	return ( strncmp( str1, str2, strlen( str2 ) ) == 0 );
}

int psio_main( FILE* fpin, FILE* fpout, int num_thisfile, int num_allfiles )
{
	char buf[PS_BUFLEN];

	/* assertions */
	assert( fpin  != NULL );
	assert( fpout != NULL );
	assert( fplog != NULL );
	assert( ary2    != NULL );
	assert( ary2[0] != NULL );

	/* initialize */
	if( !ary_flags['C'] )   /* not cat-mode */
	{
		count_page = 0;       /* reset count */
		psheader[0] = '\0';   /* erase previous header */
	}

	while( fgets( buf, numof( buf ), fpin ) )
	{
		if     ( strheadequal( buf, "%%BeginObject:" ) )
		{
			if( buf[15] == 'b' ) /* background */
			{
				/* skip background if flag is ON */
				block_else( fpin, fpout, buf, "%%EndObject", ary_flags['B'] );
			}
			else   /* grph1 */
			{
				fputs( buf, fpout );   /* %%BeginObject: grph1 */
				while( fgets( buf, numof( buf ), fpin ) )
				{
					if     ( strheadequal( buf, "%%Begin SZTN" ) && ary_flags['T'] )
					{
						block_sztn( fpin, fpout, buf );
					}
					else
					{
						/* pick up parameters */
						pickup_param1( buf );

						fputs( buf, fpout );

						if( strheadequal( buf, "%%EndObject" ) ) break;
					}
				}
			}
		}
		else if( strheadequal( buf, "%%Page" ) )
		{
			int i;

			/* count page */
			count_page++;
			fprintf( fpout, "%%%%Page: %d %d\n", count_page, count_page );
			for( i = 0; i < 5; i++ )   /* gsave ~ setmiterlimit */
			{
				if( fgets( buf, numof( buf ), fpin ) == 0 )
				{
					fprintf( fplog, "Error: Failed fgets\n" );
					return 1;
				}
				fputs( buf, fpout );
			}
			chprintf( 'P', fplog, "Page %d\n", count_page );
		}
		else if( strheadequal( buf, "grestore" ) )
		{
			fgets( buf, numof( buf ), fpin );   /* showpage */
			fputs( "grestore\n", fpout );
			fputs( "showpage\n", fpout );
			//chprintf( 'N', fplog, "\n" );   /* space */
		}
		else if( strheadequal( buf, "%!PS" ) )
		{
			/* header & definitions */
			if( psheader[0] != '\0' )   /* skip */
			{
				block_else( fpin, fpout, buf, "%%EndProlog", TRUE );
			}
			else   /* copy and save */
			{
				fputs( buf, fpout );
				assert( numof( buf ) <= numof( psheader ) );
				strncat2( psheader, buf, numof( buf ), numof( psheader ) );
				while( TRUE )
				{
					if( fgets( buf, numof( buf ), fpin ) == 0 )
					{
						fprintf( fplog, "Error: Failed fgets at prolog\n" );
						return 1;
					}
					fputs( buf, fpout );
					if( strncat2( psheader, buf, numof( buf ), numof( psheader ) ) )
					{
						fprintf( stderr, "Error: PS header is too long!\n" );
						return 1;
					}
					if( strheadequal( buf, "%%EndProlog" ) ) break;
				}
			}
		}
		else if( strheadequal( buf, "%%Trailer" ) )
		{
			/* footer(?) --> skip until last */
			fgets( buf, numof( buf ), fpin );   /* %%Pages: %d */
			fgets( buf, numof( buf ), fpin );   /* %%EOF */
			chprintf( 'P', fplog, "\n" );   /* space */
		}
		else
		{
			/* error */
			fprintf( stderr, "Error: Unexpected line!\n" );
			fprintf( stderr, "       %s\n", buf );
			return 1;
		}
	}

	/* EOF or error */
	if( !feof( fpin ) )
	{
		fprintf( stderr, "Error: Failed to read line!\n" );
		return 1;
	}

	/* append footer */
	if( !ary_flags['C'] || num_thisfile == num_allfiles )
	{
		fprintf( fpout, "%%%%Trailer\n"               );
		fprintf( fpout, "%%%%Pages: %d\n", count_page );
		fprintf( fpout, "%%%%EOF\n"                   );
	}

	return 0;
}

int block_sztn( FILE* fpin, FILE* fpout, char* buf_begin )
{
	int ipat;
	char colorinfo[64];
	BOOL exist_tone, exist_line;


	/* get IPAT */
	if( sscanf( buf_begin, "%%%%Begin SZTN [%d]", &ipat ) < 1 )
	{
		fprintf( fplog, "Error: IPAT could not be get from\n"
		                "       %s\n", buf_begin );
		return 1;
	}

	/* initialize */
	memset( colorinfo, '\0', numof( colorinfo ) );
	exist_tone = FALSE;
	exist_line = FALSE;
	mem_flush();

	/* read block all */
	while( TRUE )
	{
		char buf[16];
		int px, py;   // DON'T use intPoint for the format specification
		char op[4];

		if( fgets( buf, numof( buf ), fpin ) == 0 )
		{
			fprintf( stderr, "Error: Failed fgets in reading SZTN [%d]\n", ipat );
			return 1;
		}
		/* re-evaluate buffer string (jump from color info) */
revaluate:

		/* scan buffer */
		if( sscanf( buf, "%d %d %3s%*s", &px, &py, &op[0] ) == 3 ) /* node */
		{
			s_Node node;

			node.x = px;
			node.y = py;
			memcpy( node.op, op, numof( node.op ) );

			mem_addnode( &node );
		}
		else if( 'A' <= buf[0] && buf[0] <= 'Z' )
		{
			switch( buf[0] )
			{
			case 'N': /* newpath */
				mem_newpath();
				break;
			case 'C': /* closepath */
				mem_setpath( TRUE );
				break;
			case 'S': /* stroke */
				mem_setpath( FALSE );
				exist_line = TRUE;
				break;
			case 'F': /* fill (never pass) */
				//exist_tone = TRUE;
				//break;
			default:
				fprintf( fplog, "Error: Unexpected line in SZTN [%d] block\n"
								"       %s\n", ipat, buf );
				return 1;
			}
		}
		else if( pickup_param1( buf ) == 0 )
		{
			fputs( buf, fpout );
		}
		else if( strheadequal( buf, "% " ) ) /* color info */
		{
			if( colorinfo[0] == '\0' )
			{
				assert( numof( buf ) <= numof( colorinfo ) );
				strncat2( colorinfo, buf, numof( buf ), numof( colorinfo ) );
				while( fgets( buf, numof( buf ), fpin ) )
				{
					if( buf[0] == 'N' || buf[0] == '%' ) break;   /* end of info */
					if( strncat2( colorinfo, buf, numof( buf ), numof( colorinfo ) ) )
					{
						fprintf( stderr, "Error: Color information in SZTN [%d] is too long !", ipat );
						return 1;
					}
				}
			}
			else
			{
				while( fgets( buf, numof( buf ), fpin ) )
				{
					if( buf[0] == 'N' || buf[0] == '%' ) break;   /* end of info */
				}
			}
			exist_tone = TRUE;
			goto revaluate;   /* don't update buffer */
		}
		else if( strheadequal( buf, "%%End" ) ) /* end of block */
		{
			break;
		}
		else
		{
			fprintf( fplog, "Error: Unexpected line in SZTN [%d] block\n"
							"       %s\n", ipat, buf_begin );
			return 1;
		}
	}

	/* branch by tone and line */
	if( exist_tone )
	{
		if( exist_line )
		{
			/* error! */
			fprintf( fplog, "Error: Both tones and lines exist in SZTN [%d] block\n", ipat );
			return 1;
		}
		else
		{
			/* hard-fill (-> tonemerge) */
			int num_nodes;

			chprintf( 'N', fplog, "SZTN[%5d]:", ipat );
			num_nodes = mem_alias();                chprintf( 'N', fplog,    " %8d", num_nodes );
			mem_rsort();
			num_nodes = mem_tonemerge();            chprintf( 'N', fplog, " -> %8d", num_nodes );
			num_nodes = mem_pathopt();              chprintf( 'N', fplog, " -> %8d", num_nodes );

			fputs( buf_begin, fpout );
			num_nodes = mem_output_tone( fpout );   //chprintf( 'N', fplog, " -> %8d", num_nodes ); // same to before
			fputs( colorinfo, fpout );
			fputs( "%%End SZTN\n", fpout );
			chprintf( 'N', fplog, "\n" );
		}
	}
	else
	{
		if( exist_line )
		{
			/* soft_fill */
			int num_nodes;

			chprintf( 'N', fplog, "SZTN[%d] is soft-filled.\n", ipat );
			num_nodes = mem_alias();
			fputs( buf_begin, fpout );
			mem_output_line( fpout );
			fputs( "%%End SZTN\n", fpout );
		}
		else
		{
			/* empty */
			fputs( buf_begin, fpout );
			fputs( "%%End SZTN\n", fpout );
		}
	}

	return 0;
}

int block_else( FILE* fpin, FILE* fpout, char* buf_begin, char* str_end, BOOL skip )
{
	char buf[64];

	if( !skip ) fputs( buf_begin, fpout );

	while( TRUE )
	{
		if( fgets( buf, numof( buf ), fpin ) == 0 )
		{
			return 1;
		}

		/* pick out parameters */
		pickup_param1( buf );

		if( !skip ) fputs( buf, fpout );

		if( strheadequal( buf, str_end ) ) break;
	}

	return 0;
}
